package inria.meije.rc.sugarcubes;

public class If extends BinaryInstruction
{
    protected boolean value;
    protected JavaBooleanExpression condition;

    public If(JavaBooleanExpression cond, Instruction t, Instruction e){
        condition = cond; left = t; right = e;
    }
    public If(JavaBooleanExpression cond, Instruction t){
        this(cond,t,new Nothing());
    }

    public String toString(){ 
        if (right instanceof Nothing) return  "if "+condition+" then "+left+" end";
        if (left instanceof Nothing) return  "if "+condition+" else "+right+" end";
        return "if "+condition+" then "+left+" else "+right+" end"; 
    }
    protected Instruction rest(){ 
        if (!firstActivation) return value ? left.residual() : right.residual();
        return super.rest();
    }
    protected void firstActivation(Context context){
        value = condition.evaluate(context.currentLink());
        super.firstActivation(context);
    }
    protected byte activation(Context context){
        return value ? left.activ(context) : right.activ(context);
    }
    public void freeze(Context context){ 
        if(value)
            left.freeze(context);
        else
            right.freeze(context);
    }
    public void notifyTerminationToJava(Context context){
        if(value)
            left.notifyTerminationToJava(context);
        else
            right.notifyTerminationToJava(context);
    }
    public void notifyWarmUpToJava(Context context){
        if(value)
            left.notifyWarmUpToJava(context);
        else
            right.notifyWarmUpToJava(context);
    }
    public void notifyFreezeToJava(Context context){
        if(value)
            left.notifyFreezeToJava(context);
        else
            right.notifyFreezeToJava(context);
    }
}
